/* This file is part of Cloudy and is copyright (C)1978-2007 by Gary J. Ferland
 * For conditions of distribution and use see copyright notice in license.txt */
/*ParseCrashDo any of several tests to check that the code can crash */
#include "cddefines.h"
#include "parse.h"
/* disable warning about undefined vars being used - one of the tests shall do exactly that */
#ifdef _MSC_VER
#pragma warning( disable : 4700 )
#endif
#ifdef __INTEL_COMPILER
#pragma warning( disable : 592 )
#endif
/* disable warning about division by zero */
#ifdef _MSC_VER
#pragma warning( disable : 4756 )
#endif

#define ARR_SIZE 10

static double ar2[ARR_SIZE];

void ParseCrashDo(char *chCard)
{
	float ar1, br1;
	long int i;

	bool lgCrash = false;
	/* div by 0 to get crash as check on FP environment */
	if( nMatch("ZERO",chCard) )
	{
		fprintf(ioQQQ," I will now div by 0 to get crash.  Hold on.\n");
		fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong ....\" then there are problems.\n");
		fflush(ioQQQ);
		ar1 = (float)(1. / ZeroNum);
		fprintf(ioQQQ," I am still alive - something is wrong, result is %e\n",
			ar1);
		lgCrash = true;
	}

	/* use some undefined numbers */
	else if( nMatch("UNDE",chCard) )
	{
		if( nMatch("STAT",chCard) )
		{
			fprintf(ioQQQ," Now I will now use an undefined static variable.  Hold on.\n");
			fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong ....\" then there are problems.\n");
			fflush(ioQQQ);
			/*lint -e530 ar2 not initialized */
			ar2[0] *= 1e-10;
			/*lint +e530 ar2 not initialized */

			fprintf(ioQQQ," I am still alive - something is wrong, the result of the multiplication of undefined by 1e-10 is %e\n", ar2[0] );
			fflush(ioQQQ);
		}
		else if( nMatch("STAC",chCard) || nMatch("AUTO",chCard) )
		{
			float A_variable_which_SHOULD_be_used_uninitialized;
			fprintf(ioQQQ," Now I will now use an undefined variable off the stack.  Hold on.\n");
			fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong ....\" then there are problems.\n");
			fflush(ioQQQ);
			/*lint -e530 a not initialized */
			A_variable_which_SHOULD_be_used_uninitialized *= 1e-10f;
			/*lint +e530 a not initialized */

			fprintf(ioQQQ," I am still alive - something is wrong, the result of the multiplication of undefined by 1e-10 is %e\n", A_variable_which_SHOULD_be_used_uninitialized );
			fflush(ioQQQ);
		}
		else /* if( nMatch("HEAP",chCard) */
		{
			/* the array size must be odd to guarantee that MyMalloc will fill with float SNaN, PvH */
			float *aa = (float*)MALLOC(3*sizeof(float));
			fprintf(ioQQQ," I will now use an undefined variable off the heap obtained with malloc.  Hold on.\n");
			/* MyIsnan is guaranteed not to crash on FPE */
			if( MyIsnan( aa[1] ) )
				fprintf(ioQQQ," The malloc'ed memory was set to NaN.\n" );
			else
				fprintf(ioQQQ," The malloc'ed memory was NOT initialized by MyMalloc.\n" );
			fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong ....\" then there are problems.\n");
			fflush(ioQQQ);
			/*lint -e530 aa[1] not initialized */
			aa[1] *= 1e-10f;
			/*lint +e530 aa[1] not initialized */
			fprintf(ioQQQ," I am still alive - something is wrong, the result of the multiplication of undefined by 1e-10 is %e\n", aa[1] );
			fflush(ioQQQ);
			free( aa );
		}
		lgCrash = true;
	}

	/* make overflow to get crash as check on FP environment */
	else if( nMatch("OVER",chCard) && nMatch("LONG",chCard) )
	{ 
		long lng;
		fprintf(ioQQQ," I will now make long overflow to get crash.  Hold on.\n");
		fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong ....\" then there are problems.\n");
		fflush(ioQQQ);
		lng = (long)(LONG_MAX*sqrt(1e6) );
		fprintf(ioQQQ," I am still alive - something is wrong, the result was %li\n",
			lng);
		lgCrash = true;
	}

	/* make overflow to get crash as check on FP environment */
	else if( nMatch("OVER",chCard) )
	{ 
		ar1 = 1e-20f; 
		fprintf(ioQQQ," I will now make float overflow to get crash.  Hold on.\n");
		fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong ....\" then there are problems.\n");
		fflush(ioQQQ);
		br1 = (float)(DBL_MAX / ar1);
		fprintf(ioQQQ," I am still alive - something is wrong, the result was %e\n",
			br1);
		lgCrash = true;
	}

	/* assert impossible to get crash as check on environment */
	else if( nMatch("ASSE",chCard) )
	{ 
		fprintf(ioQQQ," I will now assert that a false statement is true to get a crash.\n\n");
		fprintf(ioQQQ," The correct behavior is for the statement \"DISASTER An assert has been thrown, this is bad\" to be printed.\n\n");
		fprintf(ioQQQ," If the next line says \"I am still alive - the assert macro is not working ....\" then there are problems.\n\n");
		fflush(ioQQQ);
		ASSERT( DBL_MAX <  ZeroNum );
		fprintf(ioQQQ," I am still alive - the assert macro is not working in this executable.\n");
		lgCrash = true;
	}

	/* assert ratios of zeros (NaN) to get crash as check on environment */
	else if( nMatch(" NAN",chCard) )
	{ 
		ar1 = 0.;
		fprintf(ioQQQ," I will now make invalid operation (div 0 by 0) to get crash.  Hold on.\n");
		fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong ....\" then there are problems.\n");
		fflush(ioQQQ);
		br1 = (float)(ar1 / ZeroNum);
		fprintf(ioQQQ," I am still alive - something is wrong, the result was %e\n",
			br1);
		lgCrash = true;
	}

	/* assert that the set_nanf routine works properly for floats */
	else if( nMatch("SETN",chCard) && nMatch("FLOA",chCard) )
	{
		float f;
		fprintf(ioQQQ," I will now initialize a float to a signaling NaN. This should never crash!\n");
		set_nanf(f);
		fprintf(ioQQQ," Initialization finished. I will now perform an operation on this variable.  Hold on.\n");
		fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong ....\" then there are problems.\n");
		fflush(ioQQQ);
		f *= 2.f;
		fprintf(ioQQQ," I am still alive - something is wrong, the result was %e\n",
			f);
		lgCrash = true;
	}

	/* assert that the set_nan routine works properly for doubles */
	else if( nMatch("SETN",chCard) )
	{
		double d;
		fprintf(ioQQQ," I will now initialize a double to a signaling NaN. This should never crash!\n");
		set_nan(d);
		fprintf(ioQQQ," Initialization finished. I will now perform an operation on this variable.  Hold on.\n");
		fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong ....\" then there are problems.\n");
		fflush(ioQQQ);
		d *= 2.;
		fprintf(ioQQQ," I am still alive - something is wrong, the result was %e\n",
			d);
		lgCrash = true;
	}

	/* array index out of bounds, two options, low and high */
	else if( nMatch("BOUN",chCard) )
	{
		bool lgEOL;
		double x;
		i = 5;

		/* read offset */
		x = FFmtRead(chCard,&i,INPUT_LINE_LENGTH,&lgEOL);
		if( lgEOL && nMatch(" LOW", chCard ) )
			x = -2.;
		if( lgEOL && nMatch("HIGH", chCard ) )
			x = 2.;

		/* if x >= 0 (which includes default case where x is no entered)
		 * i will be x beyond the end of the array, or x before the start */
		i = ( x >= 0. ) ? (long)(x+0.5) + ARR_SIZE : (long)(x-0.5);

		/* must turn off PCLint detection of logical errors in this block */
		/*lint -e662 out of bounds*/
		/*lint -e661 out of bounds*/
		if( nMatch("STAT",chCard) )
		{
			fprintf(ioQQQ," I will now access static array element ar2[%ld].  Hold on.\n", i );
			fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong\" then there are problems.\n");
			fflush(ioQQQ);
			ar2[i] = 1e-10;

			fprintf(ioQQQ," I am still alive - something is wrong\n" );
			fflush(ioQQQ);
		}
		else if( nMatch("STAC",chCard) || nMatch("AUTO",chCard) )
		{
			float a[ARR_SIZE];
			fprintf(ioQQQ," I will now access automatic array element a[%ld].  Hold on.\n", i );
			fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong\" then there are problems.\n");
			fflush(ioQQQ);
			a[i] = 1e-10f;

			fprintf(ioQQQ," I am still alive - something is wrong, return value was %.2e\n", a[i] );
			fflush(ioQQQ);
		}
		else if( nMatch("HEAP",chCard) )
		{
			int *ibound;		
			ibound = ((int *)MALLOC( ARR_SIZE*sizeof(int) ));
			fprintf(ioQQQ," I will now access malloced heap array element ibound[%ld].  Hold on.\n", i );
			fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong\" then there are problems.\n");
			fflush(ioQQQ);
			ibound[i] = 1;
			fprintf(ioQQQ," I am still alive - something is wrong, return value is %i\n" , ibound[i] );
			fflush(ioQQQ);
			free(ibound);
		}
		else
		{
			fprintf(ioQQQ," The CRASH BOUNDS command has three different tests.  One must be specified\n" );
			fprintf(ioQQQ," The HEAP option tests a malloc/'d array - this tests valgrind or purify.\n");
			fprintf(ioQQQ," The STATIC option tests a static declared array, and the STACK or AUTO option tests an automatic array - these test pgcc.\n");
			fprintf(ioQQQ," All have a number as an optional argument, the array element to be accessed.\n");
			fflush(ioQQQ);
		}
		/*lint +e662 out of bounds*/
		/*lint +e661 out of bounds*/
		lgCrash = true;
	}

	/* test the isnan function */
	else if( nMatch("ISNA",chCard) )
	{
		if( nMatch("FLOA",chCard) )
		{
			float ff;
			fprintf(ioQQQ," I will now set a float to SNaN\n" );
			set_nanf( ff );
			fprintf(ioQQQ," I will now test this variable with the isnan function\n" );
			fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong\" then there are problems.\n");
			ASSERT( !isnan( ff ) );
			fprintf(ioQQQ," I am still alive - something is wrong, value is %e\n", ff );
		}
		else /* if( nMatch("DOUB",chCard) ) */
		{
			double dd;
			fprintf(ioQQQ," I will now set a double to SNaN\n" );
			set_nan( dd );
			fprintf(ioQQQ," I will now test this variable with the isnan function\n" );
			fprintf(ioQQQ," If the next line says \"I am still alive - something is wrong\" then there are problems.\n");
			ASSERT( !isnan( dd ) );
			fprintf(ioQQQ," I am still alive - something is wrong, value is %e\n", dd );
		}
	}

	else
	{
		fprintf(ioQQQ,
			" crash options are ZERO, UNDEfined, OVERflow, ASSErt, _NAN, SETNan, BOUNds, and ISNAn.\n");
		lgCrash = true;
	}

	if( lgCrash )
	{
		puts( "[Stop in ParseCrashDo]" );
		cdEXIT(EXIT_FAILURE);
	}
}
/* turn this back on */
#ifdef _MSC_VER
#pragma warning( default : 4700 )/* disable warning that conditional expression constant*/
#endif
