/* This file is part of Cloudy and is copyright (C)1978-2007 by Gary J. Ferland
 * For conditions of distribution and use see copyright notice in license.txt */
/*ParsePrint parse the print command  */
/*prt_constants print physical constants */
#include "cddefines.h"
#include "physconst.h"
#include "rfield.h"
#include "path.h"
#include "iso.h"
#include "iterations.h"
#include "lines.h"
#include "called.h"
#include "elementnames.h"
#include "prt.h"
#include "h2.h"
#include "parse.h"
#include "version.h"
/*prt_constants print physical constants */
static void prt_constants(void);

void ParsePrint(
	/* input line was converted to caps by calling routine */
	char *chCARD_CAPS )
{
	bool lgEOL;
	int ipISO;
	long int i, 
	  j, 
	  nelem,
	  num1;
	double a;
	double sum;

	DEBUG_ENTRY( "ParsePrint()" );

	/* >>chng 01 aug 91, had been series of if branches, and could hit more than
	 * one key - dangerous!  changed to else if so only one hit per line possible */
	if( nMatch("AGES",chCARD_CAPS) )
	{
		/* print all estimates of cloud timescales */
		prt.lgPrnAges = true;
	}

	else if( nMatch("ARRA",chCARD_CAPS) )
	{
		/* print arrays for ionization balance of heavy elements */
		if( nMatch( "ONLY" , chCARD_CAPS ) )
		{
			/* returns element number on C scale */
			if( (nelem = GetElem(chCARD_CAPS))<0 )
			{
				fprintf(ioQQQ,"An element name must appear on this PRINT ARRAYS ONLY xx command.\n");
				puts( "[Stop in ParsePrint" );
				cdEXIT(EXIT_FAILURE);
			}
			/* have the element number, turn on its print */
			prt.lgPrtArry[nelem] = true;
		}
		else
		{
			/* this flag, print arrays for all elements */
			for( nelem=ipHYDROGEN; nelem<LIMELM; ++nelem )
			{
				prt.lgPrtArry[nelem] = true;
			}
		}
	}

	else if( nMatch("CITA",chCARD_CAPS) )
	{
		fprintf( ioQQQ, "\n\nCloudy is a research project that involves the creative"
			" efforts of many people.  It should be cited as follows:\n" );
		fprintf( ioQQQ, "Calculations were performed with version %s of Cloudy,"
			" last described by %s.\n\n",
			version.chVersion,
			version.chCitationShort);
		fprintf( ioQQQ, "The reference is:\n%s\n\n", version.chCitation );
		fprintf( ioQQQ, "In Latex this is:\n%s\n\n", version.chCitationLatex  );
	}

	else if( nMatch("COLU",chCARD_CAPS) && nMatch("DENS",chCARD_CAPS) )
	{
		if( nMatch(" ON ",chCARD_CAPS) )
		{
			/* print column densities of elements - this is default */
			prt.lgPrintColumns = true;
		}
		else if( nMatch(" OFF",chCARD_CAPS) )
		{
			/* print column densities of elements */
			prt.lgPrintColumns = false;
		}
	}

	/* option to print departure coefficients in addition to level pops 
	 * keywords He-like to do helike sequence element, else do h-like
	 * element name, if not recognized, does hydrogen
	 * so with no options prints hydrogen itself */
	else if( nMatch("DEPA",chCARD_CAPS) )
	{
		if( nMatch("HE-L",chCARD_CAPS) )
		{
			ipISO = ipHELIUM;
		}
		else
		{
			ipISO = ipHYDROGEN;
		}

		/* now check for element name */
		nelem = GetElem( chCARD_CAPS );
		/* return value is < 0 if no element recognized - in this case use H */
		nelem = MAX2(0,nelem);

		/* if no element specified but he-like iso, then use helium */
		if( nelem==0 && ipISO==ipHE_LIKE )
			nelem = ipHELIUM;

		if( nelem < ipISO )
		{
			fprintf(ioQQQ,"This iso-sequence (%s) and element (%s) are impossible.\n",
				elementnames.chElementName[ipISO],
				elementnames.chElementName[nelem]);
			puts( "[Stop in ParsePrint" );
			cdEXIT(EXIT_FAILURE);
		}

		/* print departure coefficients instead of hydrogen level populations */
		iso.lgPrtDepartCoef[ipISO][nelem] = true;
	}

	else if( nMatch("CONS",chCARD_CAPS) )
	{
		/* print physical constants, routine is below */
		prt_constants();
	}

	else if( nMatch("ERRO",chCARD_CAPS) )
	{
		/* print errors to special window */
		lgPrnErr = true;
	}

	else if( nMatch("HEAT",chCARD_CAPS) )
	{
		/* print heat arrays */
		prt.lgPrintHeating = true;
	}

	else if( nMatch("PATH",chCARD_CAPS) )
	{
		/* print the path */
		fprintf(ioQQQ,"The path ");
		if( lgDataPathSet )
		{
			fprintf(ioQQQ,"is ->**%s**<-\n",chDataPath );
		}
		else
		{
			fprintf(ioQQQ,"has not been set.\n;" );
		}
	}

	/*else if( nMatch("H-LI",chCARD_CAPS))*/
	else if( nMatch("POPU",chCARD_CAPS))
	{
		if( nMatch("HE-L",chCARD_CAPS) )
		{
			ipISO = ipHELIUM;
		}
		else
		{
			ipISO = ipHYDROGEN;
		}

		/* now check for element name */
		nelem = GetElem( chCARD_CAPS );
		/* return value is < 0 if no element recognized - in this case use H */
		nelem = MAX2(0,nelem);

		/* if no element specified but he-like iso, then use helium */
		if( nelem==0 && ipISO==ipHE_LIKE )
			nelem = ipHELIUM;

		if( nelem < ipISO )
		{
			fprintf(ioQQQ,"This iso-sequence (%s) and element (%s) are impossible.\n",
				elementnames.chElementName[ipISO],
				elementnames.chElementName[nelem]);
			puts( "[Stop in ParsePrint" );
			cdEXIT(EXIT_FAILURE);
		}

		/* print hydrogenic H-like level populations */
		iso.lgPrtLevelPops[ipISO][nelem] = true;

		/* option to set number of levels to print */
		i = 5;
		iso.numPrintLevels[ipISO][nelem] = (long)FFmtRead(chCARD_CAPS,&i,INPUT_LINE_LENGTH,&lgEOL);
		if( lgEOL )
		{
			/* no number on line so use default, note that the current
			 * value of iso.numLevels_max[ipH_LIKE][nelem] may be the default, or have been reset
			 * with hydrogenic command.  So the effects of the following
			 * will depend on the order of the commands, if both this command
			 * and hydrogenic levels occur in the same input stream */
			iso.numPrintLevels[ipISO][nelem] = iso.numLevels_max[ipISO][nelem] - 1;
		}
		/* make sure at least three, since we will assert such in PrtHydro */
		if( iso.numPrintLevels[ipISO][nelem] < 3 )
		{
			fprintf(ioQQQ,"Sorry, I simply must print at least three levels.\n");
			iso.numPrintLevels[ipISO][nelem] = 3;
		}
		ASSERT( iso.numPrintLevels[ipISO][nelem] < iso.numLevels_max[ipISO][nelem] );
	}

	/* option to only print last iteration */
	else if( nMatch("LAST",chCARD_CAPS) )
	{
		prt.lgPrtLastIt = true;
	}

	/* the print line command as several options */
	else if( nMatch("LINE",chCARD_CAPS) )
	{
		if( nMatch(" ALL",chCARD_CAPS) )
		{
			/* turn on all printed components */
			prt.lgPrnPump = true;
			prt.lgPrnColl = true;
			prt.lgPrnHeat = true;
		}

		else if( nMatch("COLL",chCARD_CAPS) )
		{
			/* also print collisional contributions */
			prt.lgPrnColl = true;
		}

		else if( nMatch("COLU",chCARD_CAPS) )
		{
			/* option to print main line array as a single column */
			prt.lgPrtLineArray = false;
			/* this also has an option - liNEAR - to print linear quantity 
			 * in exponential format */
			if( nMatch("NEAR",chCARD_CAPS) )
			{
				prt.lgPrtLineLog = false;
			}
		}

		/* force printing emergent intensities */
		else if( nMatch("EMER",chCARD_CAPS) && nMatch("GENT",chCARD_CAPS) )
		{
			prt.lgPrtLineEmergent = true;
			LineSave.lgLineEmergent = true;
		}

		else if( nMatch("FAIN",chCARD_CAPS) && !(nMatch("OPTI",chCARD_CAPS)&&nMatch("DEPT",chCARD_CAPS)) )
		{
			/* print line faint - above do not trigger on optical depth 
			 * option to adjust intensity of faintest line to print */
			/* >> 01 feb 01, move print faint into print line faint */
			/* faintest line, rel to norm line, to print; either linear of log */
			i = 5;
			a = FFmtRead(chCARD_CAPS,&i,INPUT_LINE_LENGTH,&lgEOL);

			/* option for, if no number, keyword=" OFF", to print all lines */
			if( lgEOL )
			{
				if( nMatch(" OFF",chCARD_CAPS) )
				{
					prt.lgFaintOn = false;
				}
				else
				{
					fprintf( ioQQQ, 
						" There faintest line to print must be on this line, sorry.\n" );
					puts( "[Stop in ParsePrint" );
					cdEXIT(EXIT_FAILURE);
				}
			}

			prt.lgFntSet = true;
			if( a <= 0. )
			{
				prt.TooFaint = (float)pow(10.,a);
			}
			else
			{
				prt.TooFaint = (float)a;
			}
		}

		else if( nMatch("FLUX",chCARD_CAPS) && nMatch("EART",chCARD_CAPS))
		{
			/* print line flux seen at earth */
			prt.lgPrintFluxEarth = true;
		}

		else if( nMatch(" H2",chCARD_CAPS) && nMatch("ELEC",chCARD_CAPS) )
		{
			/* print H2 electronic lines too - -1 since number of electronic
			 * levels is not yet known, will set when H2 actually called */
			h2.nElecLevelOutput = -1;
		}

		else if( nMatch("HEAT",chCARD_CAPS) )
		{
			/* also print heating contributions */
			prt.lgPrnHeat = true;
		}

		else if( nMatch("INWA",chCARD_CAPS) )
		{
			/* also print inward contributions */
			prt.lgPrnInwd = true;
		}

		else if( nMatch("OPTI",chCARD_CAPS) && nMatch("DEPT",chCARD_CAPS) )
		{
			/* print line optical depths, with option for smallest to print */
			if( nMatch(" OFF",chCARD_CAPS) )
			{
				/* turn off or on printing of optical depths - default off */
				prt.lgPrtTau = false;
			}
			else
			{
				prt.lgPrtTau = true;
			}
			if( nMatch("FAIN",chCARD_CAPS) )
			{
				/* log of faintest optical depth */
				i = 5;
				prt.PrtTauFnt = (float)pow(10.,FFmtRead(chCARD_CAPS,&i,INPUT_LINE_LENGTH,&lgEOL));
				if( lgEOL )
				{
					fprintf( ioQQQ, " There must be a number for the FAINT option.  They are HEAD and ZONE.  Sorry.\n" );
					puts( "[Stop in ParsePrint" );
					cdEXIT(EXIT_FAILURE);
				}
			}
		}

		else if( nMatch("PUMP",chCARD_CAPS) )
		{
			/* also print pump contributions */
			prt.lgPrnPump = true;
		}

		else if( nMatch("SORT",chCARD_CAPS) )
		{
			/* >>chng 01 aug 18, print sort command works after all these years,
			 * sort by wavelength or intensity */
			/* turn on sorting with respect to wavelength */
			prt.lgSortLines = true;
			if( nMatch("WAVE",chCARD_CAPS) )
			{
				/* sort by wavelength */
				/* remember which one to do */
				prt.lgSortLineIntensity = false;
				prt.lgSortLineWavelength = true;

				/* wavelength has range option */
				/* option to only certain print range of lines */
				if( nMatch("RANG",chCARD_CAPS) )
				{
					i = 5;
					prt.wlSort1 = (float)FFmtRead(chCARD_CAPS,&i,INPUT_LINE_LENGTH,&lgEOL);
					/* line was entered, look for possible micron or cm label */
					if( chCARD_CAPS[i-1] == 'M' )
					{
						/* microns */
						prt.wlSort1 *= 1e4;
					}
					else if( chCARD_CAPS[i-1] == 'C' )
					{
						/* microns */
						prt.wlSort1 *= 1e8;
					}
					prt.wlSort2 = (float)FFmtRead(chCARD_CAPS,&i,INPUT_LINE_LENGTH,&lgEOL);
					/* line was entered, look for possible micron or cm label */
					if( chCARD_CAPS[i-1] == 'M' )
					{
						/* microns */
						prt.wlSort2 *= 1e4;
					}
					else if( chCARD_CAPS[i-1] == 'C' )
					{
						/* microns */
						prt.wlSort2 *= 1e8;
					}
					if( lgEOL )
					{
						fprintf( ioQQQ, " There must be two numbers for the RANGE option, the lower and upper wavelength.  Sorry.\n" );
						puts( "[Stop in ParsePrint" );
						cdEXIT(EXIT_FAILURE);
					}
					if( prt.wlSort1 <0. || prt.wlSort2 <0. || 
						prt.wlSort1 >= prt.wlSort2 )
					{
						fprintf( ioQQQ, " The lower and upper wavelength must be positive and in the correct order.  Sorry.\n" );
						puts( "[Stop in ParsePrint" );
						cdEXIT(EXIT_FAILURE);
					}
				}
				else
				{
					prt.wlSort1 = -1;
					prt.wlSort2 = 1e30f;
				}
			}
			else if( nMatch("INTE",chCARD_CAPS) )
			{
				/* sort by intensity/luminosity */
				/* remember which one to do */
				prt.lgSortLineIntensity = true;
				prt.lgSortLineWavelength = false;
			}
			else
			{
				fprintf( ioQQQ, "I can sort by wavelength or intensity - one must be specified.\nSorry.\n" );
				puts( "[Stop in ParsePrint" );
				cdEXIT(EXIT_FAILURE);
			}

		}
		else if( nMatch(" SUM",chCARD_CAPS) )
		{
			/* option to read in set of lines to sum over */
			sum = PrtLineSum( "READ" );
			/* sum is not used anywhere, following makes lint shut up */
			if( false ) fprintf(ioQQQ,"%.2e\n", sum);
		}

		else if( nMatch("SURF",chCARD_CAPS) && nMatch("BRIG",chCARD_CAPS) )
		{
			/* print surface brightness rather than 4pi J */
			prt.lgSurfaceBrightness = true;
			/* default is per sr, arcsec option changes to sq arcsec */
			if( nMatch("ARCS",chCARD_CAPS ) )
			{
				/* use sr */
				prt.lgSurfaceBrightness_SR = false;
			}
			else
			{
				/* use sq arcsec */
				prt.lgSurfaceBrightness_SR = true;
			}
		}
		else
		{
			fprintf( ioQQQ, "One of the keys should have appeared.  \nPlease consult Hazy.\nSorry.\n" );
			puts( "[Stop in ParsePrint" );
			cdEXIT(EXIT_FAILURE);
		}
	}

	/* print maser lines when TAV is called */
	else if( nMatch("MASE",chCARD_CAPS) )
	{
		prt.lgPrtMaser = true;
	}

	else if( nMatch("ONLY",chCARD_CAPS) )
	{
		if( nMatch("ZONE",chCARD_CAPS) )
		{
			prt.lgOnlyZone = true;
		}

		else if( nMatch("HEAD",chCARD_CAPS) )
		{
			prt.lgOnlyHead = true;
		}

		else
		{
			fprintf( ioQQQ, " There must be a keyword for the ONLY option.  They are HEAD and ZONE.  Sorry.\n" );
			puts( "[Stop in ParsePrint" );
			cdEXIT(EXIT_FAILURE);
		}
	}

	else if( nMatch("STAR",chCARD_CAPS) )
	{
		/* start printout at specified zone */
		called.lgTalk = false;
		prt.lgPrtStart = true;
		i = 5;
		prt.nstart = (long int)FFmtRead(chCARD_CAPS,&i,INPUT_LINE_LENGTH,&lgEOL);
		if( lgEOL )
		{
			fprintf( ioQQQ, 
				" The zone on which the print is to start MUST be entered on this line.  Sorry.\n" );
			puts( "[Stop in ParsePrint" );
			cdEXIT(EXIT_FAILURE);
		}
	}

	/* print continuum command */
	else if( nMatch("CONT",chCARD_CAPS) )
	{
		/* >>chng 01 jun 30, logic of keys swapped around, so no keyword gets nFnu info */
		/* print continuum diffuse turns on nFnu continuum at many energies */
		if( nMatch("BLOC",chCARD_CAPS) )
		{
			/* option to print emergent continuum at end of calculation*/
			prt.lgPrtCont = true;
		}
		else if( nMatch("INDI" , chCARD_CAPS ))
		{
			/* option to print lines and continuum that go into each continuum 
			 * index the continuum index is the cell within the continuum 
			 * array - this identifies lines that occur within each 
			 * continuum cell */
			prt.lgPrtContIndices = true;
			/* these are lower and upper limits to the energy range in Rydbergs.
			* they are the first and second number on the command line, lower and
			* upper bounds of the code are used if not specified */
			i = 5;
			/* if no number on line then zero is returned, this is fine, since
			 * we want the lower energy bound of the code */
			prt.lgPrtContIndices_lo_E = (float)FFmtRead(chCARD_CAPS,&i,INPUT_LINE_LENGTH,&lgEOL);
			prt.lgPrtContIndices_hi_E = (float)FFmtRead(chCARD_CAPS,&i,INPUT_LINE_LENGTH,&lgEOL);
			/* if we hit end of line then use high-energy limit of code - that is,
			 * include all energies */
			if( lgEOL )
				prt.lgPrtContIndices_hi_E = (float)rfield.egamry;
		}
		else
		{
			/* option to print continuum points within emission lines block */
			prt.lgPrnDiff = true;
		}
	}

	else if( nMatch("COOL",chCARD_CAPS) )
	{
		/* print cooling array for a specified one */
		i = 5;
		prt.nzdump = (long int)FFmtRead(chCARD_CAPS,&i,INPUT_LINE_LENGTH,&lgEOL);

		/* dump all zones if argument is zero or not present  */
		if( lgEOL )
		{
			prt.nzdump = 0;
		}
	}

	else if( nMatch("QUIE",chCARD_CAPS) || (nMatch(" OFF",chCARD_CAPS) && 
		!nMatch("FAIN" ,chCARD_CAPS)) )
	{
		/* in above, there is also a 'print faint off' command
		 * QUIET or OFF means turn off printout */
		called.lgTalk = false;
	}

	else if( nMatch(" ON ",chCARD_CAPS) )
	{
		/* on means turn on printout, lgTalkIsOK is set true in cdInit, but set
		 * false in dooptimize.  this keeps printout quiet during optimize,
		 * even when init files are parsed */
		/* called.lgTalkForcedOff was set true with cdTalk(false), if this was
		 * set then do not respect this command.  this is to prevent print on at end
		 * of init file from turning on print in grids when print is turned off */
		if( called.lgTalkIsOK && !called.lgTalkForcedOff )
		{
			called.lgTalk = true;
		}
	}

	else if( nMatch("SHOR",chCARD_CAPS) )
	{
		/* make short printout, don't print last */
		prt.lgPrtShort = true;
		if( !prt.lgFntSet )
			prt.TooFaint = 0.001f;
	}

	else if( nMatch("EVER",chCARD_CAPS) )
	{
		/* print every nth zone */
		i = 5;
		num1 = (long int)FFmtRead(chCARD_CAPS,&i,INPUT_LINE_LENGTH,&lgEOL);
		if( lgEOL )
		{
			fprintf( ioQQQ, " The number of zones to print MUST be entered on this line.  Sorry.\n" );
			puts( "[Stop in ParsePrint" );
			cdEXIT(EXIT_FAILURE);
		}

		iterations.IterPrnt[0] = MAX2(num1,1);

		for( j=1; j < iterations.iter_malloc; j++ )
		{
			iterations.IterPrnt[j] = (long int)FFmtRead(chCARD_CAPS,&i,INPUT_LINE_LENGTH,&lgEOL);
			if( lgEOL )
			{
				iterations.IterPrnt[j] = iterations.IterPrnt[j-1];
			}
		}
	}

	/* check if no keywords were recognized. */
	else
	{
		fprintf( ioQQQ, " There MUST be a keyword on the following line.  Sorry.\n" );
		fprintf( ioQQQ, " The PRINT FAINT command is now the PRINT LINE FAINT command.\n" );
		fprintf( ioQQQ, " %80.80s\n", chCARD_CAPS );
		puts( "[Stop in ParsePrint]" );
		cdEXIT(EXIT_FAILURE);
	}

	DEBUG_EXIT( "ParsePrint()" );
	return;
}

/*prt_constants print physical and machine constants */
static void prt_constants(void)
{

	DEBUG_ENTRY( "prt_constants()" );

	fprintf(ioQQQ,"\n\nPhysical constants used by Cloudy, taken from physconst.h\n");

	fprintf(ioQQQ,"E\t%g\n",E);
	fprintf(ioQQQ,"EULER\t%g\n",EULER);
	fprintf(ioQQQ,"PI\t%g\n",PI);
	fprintf(ioQQQ,"PI2\t%g\n",PI2);
	fprintf(ioQQQ,"PI4\t%g\n",PI4);
	fprintf(ioQQQ,"PI8\t%g\n",PI8);
	fprintf(ioQQQ,"SQRT2\t%g\n",SQRT2);
	fprintf(ioQQQ,"SQRTPI\t%g\n",SQRTPI);
	fprintf(ioQQQ,"SQRTPIBY2\t%g\n",SQRTPIBY2);
	fprintf(ioQQQ,"LN_TWO\t%g\n",LN_TWO);
	fprintf(ioQQQ,"LN_TEN\t%g\n",LN_TEN);
	fprintf(ioQQQ,"LOG10_E\t%g\n",LOG10_E);
	fprintf(ioQQQ,"OPTDEP2EXTIN\t%g\n",OPTDEP2EXTIN);
	fprintf(ioQQQ,"RADIAN\t%g\n",RADIAN);
	fprintf(ioQQQ,"SOLAR_MASS\t%g\n",SOLAR_MASS);
	fprintf(ioQQQ,"SOLAR_LUMINOSITY\t%g\n",SOLAR_LUMINOSITY);
	fprintf(ioQQQ,"AU\t%g\n",AU);
	fprintf(ioQQQ,"ATOMIC_MASS_UNIT\t%g\n",ATOMIC_MASS_UNIT);
	fprintf(ioQQQ,"ELECTRON_MASS\t%g\n",ELECTRON_MASS);
	fprintf(ioQQQ,"PROTON_MASS\t%g\n",PROTON_MASS);
	fprintf(ioQQQ,"BOLTZMANN\t%g\n",BOLTZMANN);
	fprintf(ioQQQ,"SPEEDLIGHT\t%g\n",SPEEDLIGHT);
	fprintf(ioQQQ,"HPLANCK\t%g\n",HPLANCK);
	fprintf(ioQQQ,"ELEM_CHARGE\t%g\n",ELEM_CHARGE);
	fprintf(ioQQQ,"RYD_INF\t%g\n",RYD_INF);
	fprintf(ioQQQ,"HIONPOT\t%g\n",HIONPOT);
	fprintf(ioQQQ,"PARSEC\t%g\n",PARSEC);
	fprintf(ioQQQ,"H_BAR \t%g\n",H_BAR );
	fprintf(ioQQQ,"ELEM_CHARGE_ESU \t%g\n",ELEM_CHARGE_ESU );
	fprintf(ioQQQ,"ELECTRIC_CONST\t%g\n",ELECTRIC_CONST);
	fprintf(ioQQQ,"HION_LTE_POP\t%g\n",HION_LTE_POP);
	fprintf(ioQQQ,"SAHA\t%g\n",SAHA);
	fprintf(ioQQQ,"ERG1CM\t%g\n",ERG1CM);
	fprintf(ioQQQ,"T1CM\t%g\n",T1CM);
	fprintf(ioQQQ,"WAVNRYD\t%g\n",WAVNRYD);
	fprintf(ioQQQ,"RYDLAM\t%g\n",RYDLAM);
	fprintf(ioQQQ,"EN1RYD\t%g\n",EN1RYD);
	fprintf(ioQQQ,"TE1RYD\t%g\n",TE1RYD);
	fprintf(ioQQQ,"EVDEGK\t%g\n",EVDEGK);
	fprintf(ioQQQ,"EVRYD\t%g\n",EVRYD);
	fprintf(ioQQQ,"EN1EV\t%g\n",EN1EV);
	fprintf(ioQQQ,"FR1RYD\t%g\n",FR1RYD);
	fprintf(ioQQQ,"HNU3C2\t%g\n",HNU3C2);
	fprintf(ioQQQ,"FR1RYDHYD\t%g\n",FR1RYDHYD );
	fprintf(ioQQQ,"HBAReV\t%g\n",HBAReV );
	fprintf(ioQQQ,"RYDLAMHYD\t%g\n",RYDLAMHYD );
	fprintf(ioQQQ,"STEFAN_BOLTZ\t%g\n",STEFAN_BOLTZ);
	fprintf(ioQQQ,"FREQ_1EV\t%g\n",FREQ_1EV);
	fprintf(ioQQQ,"FINE_STRUCTURE\t%g\n",FINE_STRUCTURE);
	fprintf(ioQQQ,"BOHR_RADIUS_CM\t%g\n",BOHR_RADIUS_CM);
	fprintf(ioQQQ,"TWO_PHOT_CONST\t%g\n",TWO_PHOT_CONST);
	fprintf(ioQQQ,"COLL_CONST\t%g\n",COLL_CONST);
	fprintf(ioQQQ,"MILNE_CONST\t%g\n",MILNE_CONST);
	fprintf(ioQQQ,"TRANS_PROB_CONST\t%g\n",TRANS_PROB_CONST);
	fprintf(ioQQQ,"\n");

	fprintf(ioQQQ,"Some other interesting sizes.\n");
	fprintf(ioQQQ,"bool\t%lu\n",(unsigned long)sizeof(bool));
	fprintf(ioQQQ,"char\t%lu\n",(unsigned long)sizeof(char));
	fprintf(ioQQQ,"int\t%lu\n",(unsigned long)sizeof(int));
	fprintf(ioQQQ,"long int\t%lu\n",(unsigned long)sizeof(long int));
	fprintf(ioQQQ,"unsigned int\t%lu\n",(unsigned long)sizeof(unsigned int));
	fprintf(ioQQQ,"float\t%lu\n",(unsigned long)sizeof(float));
	fprintf(ioQQQ,"double\t%lu\n",(unsigned long)sizeof(double));
	fprintf(ioQQQ,"double*\t%lu\n",(unsigned long)sizeof(double*));
	fprintf(ioQQQ,"double**\t%lu\n",(unsigned long)sizeof(double**));
	fprintf(ioQQQ,"double***\t%lu\n",(unsigned long)sizeof(double***));
	fprintf(ioQQQ,"double****\t%lu\n",(unsigned long)sizeof(double****));
	fprintf(ioQQQ,"\n");

	fprintf(ioQQQ,"Some constants from float.h.\n");
	/* some constants from float.h */
	fprintf(ioQQQ,"DBL_DIG \t%i\n", DBL_DIG);         /* # of decimal digits of precision */
	fprintf(ioQQQ,"DBL_EPSILON \t%g\n",DBL_EPSILON);   /* smallest such that 1.0+DBL_EPSILON != 1.0 */
	fprintf(ioQQQ,"DBL_MANT_DIG\t%i\n",DBL_MANT_DIG); /* # of bits in mantissa */
	fprintf(ioQQQ,"DBL_MAX\t%g\n", DBL_MAX);           /* max value */
	fprintf(ioQQQ,"DBL_MAX_10_EXP\t%i\n", DBL_MAX_10_EXP); /* max decimal exponent */
	fprintf(ioQQQ,"DBL_MAX_EXP\t%i\n", DBL_MAX_EXP);  /* max binary exponent */
	fprintf(ioQQQ,"DBL_MIN\t%g\n", DBL_MIN);           /* min positive value */

	fprintf(ioQQQ,"FLT_DIG\t%i\n", FLT_DIG);          /* # of decimal digits of precision */
	fprintf(ioQQQ,"FLT_EPSILON\t%g\n", FLT_EPSILON);   /* smallest such that 1.0+FLT_EPSILON != 1.0 */
	fprintf(ioQQQ,"FLT_MANT_DIG\t%i\n", FLT_MANT_DIG); /* # of bits in mantissa */
	fprintf(ioQQQ,"FLT_MAX\t%g\n", FLT_MAX);            /* max value */
	fprintf(ioQQQ,"FLT_MAX_10_EXP\t%i\n", FLT_MAX_10_EXP);/* max decimal exponent */
	fprintf(ioQQQ,"FLT_MAX_EXP\t%i\n", FLT_MAX_EXP);   /* max binary exponent */
	fprintf(ioQQQ,"FLT_MIN\t%g\n", FLT_MIN);            /* min positive value */

	fprintf(ioQQQ,"\n\n\n");

	DEBUG_EXIT( "prt_constants()" );

	return;
}
