/* This file is part of Cloudy and is copyright (C)1978-2007 by Gary J. Ferland
 * For conditions of distribution and use see copyright notice in license.txt */
/*Zerologic set initial values for logical variables 
 * this is called before any commands are parsed */
#ifdef _MSC_VER
#	pragma warning( disable : 4305 )/* disable const double to float warning in MS VS - 
									   very large number of warns result */
#endif
#include "cddefines.h"
#include "map.h"
#include "optimize.h"
#include "prt.h"
#include "extinc.h"
#include "plot.h"
/*  */
/* following used to enter total predicted continuum into emission line array 
 * actually entered into line array in lineset1
 */
#include "predcont.h"
/* energies where diffuse continuum is to be entered into line array 
 * variables are declared in predcont.h, where array dimensioned 
 * NB - if these numbers change, the wavelength in the printout will change 
 * too, since the wavelength is derived form this */
/* >>>chng 99 mar 23, adjusted energies so that wavelength line list is
 * the same as it was in C90 - small changes were caused by going over
 * to proper Rydberg constant */
float EnrPredCont[NPREDCONT] = {
	/* this is radio continuum at 3.4 cm */
	  2.680e-06
	, 7.445e-04 
	, 1.498e-03 
	, 2.211e-03 
	, 2.952e-03 
	, 3.677e-03 
	, 3.7501e-03 /* Ney-Allen */
	, 3.9915e-03 /* Ney-Allen */
	, 4.2543e-03 /* Ney-Allen */
	, 4.314e-03 
	, 4.6446e-03 /* Ney-Allen */
	, 5.162e-03 
	, 5.2462e-03 /* Ney-Allen */
	, 5.8079e-03 /* Ney-Allen */
	, 6.240e-03 
/* 	, 7.320e-03  */
	, 7.3312e-03 /* Ney-Allen */
	, 7.9936e-03 /* Ney-Allen */
	, 8.7119e-03 /* Ney-Allen */
	, 9.6125e-03 /* Ney-Allen */
	, 9.77243e-03
	, 1.1099e-02 /* Ney-Allen */
	, 1.2022e-02 /* Ney-Allen */
	, 1.29253e-02 
	, 2.2152e-02 
	, 3.92044e-02 
	, 5.54593e-02 
	/* next two either side of n=4 edge of hydrogen, set to 1.5% off either direction*/
	/* >>chng 00 sep 18, had been too close in energy */
	, 6.1563e-02 
	, 6.3437e-02 
	, 8.1460e-02 
	/* >>chng 00 sep 14, changed energies of paschen jump to be farther away as
	 * per note on BJ */
	, 0.1094 
	, 0.1128 
	, 0.14675 
	, 0.18653 
	/* >>chng 00 sep 14, next two energies changed since they were too close to BJ
	 * and so both ended up shortward of limit*/
	, 0.246   /* these two are the Balmer jump, below and above. */
	, 0.254  /* continuum binning not much better than 1% so offset energies by more */
	, 0.375  /* peak on two photon continuum */
	, 0.38096 
	, 0.43994 
	, 0.44394 
	, 0.50811 
	, 0.57489
	, 0.62487 
	, 0.67155 
	, 0.70244 
	, 0.72163 
	, 0.74812 
	, 0.76172 
	, 0.77551 
	, 0.79681 
	, 0.81859 
	, 0.8260 
	, 0.84859 
	, 0.85618 
	, 0.87967 
	, 0.911267 /*exactly 1000A */ 
	/* points on either side of Lyman jump,
	 * energies changed to be robust when energy grid changes,
	 * grid resolution is about 1%, so change from 0.99783 and 1.000
	 * to 1 +/- 1.5%
	 * >>chng 00 sep 23 change wavelength points for next two */
	, 0.985 
	, 1.015 
	, 1.199 
	, 1.299 
	, 1.4984 
	, 1.58441 
	/* points on either side of Lyman jump,
	 * energies changed to be robust when energy grid changes,
	 * grid resolution is about 1%, so change from 1.80433 and 1.809
	 * to 1.807 +/- 1.5%
	 * >>chng 00 sep 23 change wavelength points for next two */
	, 1.780 
	, 1.834 
	, 2.283};

long int ipPredCont[NPREDCONT];
#include "heavy.h"
#include "fudgec.h"
#include "noexec.h"
#include "reason.h"
#include "rt.h"
#include "trace.h"
#include "magnetic.h"
#include "hextra.h"
#include "zero.h"

void Zerologic(void)
{
	long int i,
		nelem;

	DEBUG_ENTRY( "Zerologic()" );

	/* parameters having to do with thermal map */
	map.RangeMap[0] = 10.;
	map.RangeMap[1] = .99e10;
	/* >>chng 04 jun 16, had been 9999, was accidentally hit once,
	 * reset to -1 to indicate not to be used */
	/*map.MapZone = 9999;*/
	map.MapZone = -1;
	map.lgMapOK = true;
	map.lgMapDone = false;
	map.nMapStep = 20;
	/* will be reset to positive number when map actually done */
	map.nMapAlloc = 0;
	map.nmap = 0;
	map.lgMapBeingDone = false;

	/* constant for the extinguish command */
	extinc.excolm = 0.;
	extinc.exleak = 0.;
	extinc.exlow = 1.;
	/* the constant that multiplies the column density to get optical depth at 1 Ryd */
	extinc.cnst_col2optdepth = 6.22e-18f;
	/* the power on the energy */
	extinc.cnst_power = -2.43f;

	/* list of shells, 1 to 7 */
	strcpy( Heavy.chShell[0], "1s" );
	strcpy( Heavy.chShell[1], "2s" );
	strcpy( Heavy.chShell[2], "2p" );
	strcpy( Heavy.chShell[3], "3s" );
	strcpy( Heavy.chShell[4], "3p" );
	strcpy( Heavy.chShell[5], "3d" );
	strcpy( Heavy.chShell[6], "4s" );

	/* zero out diffuse recombination emission arrays */
	for( i=0; i < LIMELM; i++ )
	{
		int j;
		for( j=0; j < (LIMELM - 1); j++ )
		{
			Heavy.xLyaHeavy[i][j] = 0.;
		}
	}

	noexec.lgNoExec = false;

	/* default no printing of optical depths, TooFaint is .1 */
	prt.lgPrtTau = false;
	prt.PrtTauFnt = 0.1;
	prt.lgPrtShort = false;
	prt.TooFaint = 1e-3;
	prt.lgFaintOn = true;
	prt.lgFntSet = false;

	/* if true then print main block of lines as array,
	 * set false with print lines column, will then
	 * do a single column of lines */
	prt.lgPrtLineArray = true;

	/* when printing a column this is option to print linear rather than log */
	prt.lgPrtLineLog = true;

	/* print emergent intensities even when sphere, set with print line emergenet */
	prt.lgPrtLineEmergent = false;

	/* print ages */
	prt.lgPrnAges = false;

	/* print column densities */
	prt.lgPrintColumns = true;

	/* option to sort lines by wavelength, print sort command */
	prt.lgSortLines = false;

	prt.lgPrtMaser = false;
	prt.lgPrintTime = true;
	
	prt.lgPrtContIndices = false;
	prt.lgPrtCont = false;
	prt.lgPrnDiff = false;
	prt.lgPrnPump = false;
	prt.lgPrnInwd = false;
	prt.lgPrnColl = false;
	prt.lgPrnHeat = false;
	/* >>chng 00 dec 08, these determine the standard items included in "nFnu", PvH */
	prt.lgSourceReflected = true;
	prt.lgSourceTransmitted = false;
	prt.lgDiffuseInward = true;
	prt.lgDiffuseOutward = true;
	prt.lgPrtLastIt = false;
	prt.lgOnlyZone = false;
	prt.lgOnlyHead = false;
	prt.lgPrtStart = false;
	prt.nstart = 0;

	/* turn off printing of heating agents */
	prt.lgPrintHeating = false;

	/* flag saying to print all matrix elements for ionization balance 
	 * set with PRINT ARRAYS command */
	for( nelem=ipHYDROGEN; nelem<LIMELM; ++nelem )
	{
		prt.lgPrtArry[nelem] = false;
	}

	/* print line flux at earth */
	prt.lgPrintFluxEarth = false;

	/* print line surface brightness, def sr, option arcsec */
	prt.lgSurfaceBrightness = false;
	prt.lgSurfaceBrightness_SR = true;

	prt.nzdump = -100;

	fudgec.nfudge = 0;

	for( i=0; i < NFUDGC; i++ )
	{
		fudgec.fudgea[i] = 0.;
	}

	hextra.TurbHeat = 0.;
	/* >>chng 01 feb 22, had set this to 1e30, change to 0 as sentinel that is not used */
	hextra.turrad = 0.;
	hextra.turback = 0.;
	hextra.cryden = 0.;
	hextra.cryden_ov_background = 0.;
	hextra.lg_CR_B_equipartition = false;
	hextra.cr_energydensity = 0;
	/* default cosmic ray background */
	/* >>chng 99 jun 24, slight change to value
	 * quoted by 
	 * >>refer	cosmic ray	ionization rate	McKee, C.M., 1999, astro-ph 9901370
	 * this will produce a total
	 * secondary ionization rate of 2.5e-17 s^-1, as tested in 
	 * test suite cosmicray.in.  If each ionization produces 2.4 eV of heat,
	 * the background heating rate should be 9.6e-29 * n*/
	/* >>chng 04 jan 26, update cosmic ray ionization rate for H0 to
	 * >>refer	cosmic ray	ionization	Williams, J.P., Bergin, E.A., Caselli, P., 
	 * >>refercon	Myers, P.C., & Plume, R. 1998, ApJ, 503, 689,
	 * H0 ionization rate of 2.5e-17 s-1 and a H2 ionization rate twice this
	 * >>chng 04 mar 15, comment said 2.5e-17 which is correct, but code produce 8e-17,
	 * fix back to correct value 
	 */
	/* NB - the rate is derived from the density.  these two are related by the secondary
	 * ionization efficiency problem.  background_rate is only here to provide the relationship
	 * for predominantly neutral gas.  the background_density is the real rate. 
	hextra.background_density = 1.99e-9f;*/
	/* >>chng 05 apr 16, to get proper ionization rate in ism_set_cr_rate, where
	 * H is forced to be fully atomic, no molecules, density from 1.99 to 2.15 */
	hextra.background_density = 2.15e-9f;
	hextra.background_rate = 2.5e-17f;
	hextra.crpowr = 0.;
	hextra.crtemp = 0.;

	/* parameters having to do with magnetic field */
	Magnetic_init();

	strncpy( reason.chReason, "reason not specified.", sizeof(reason.chReason) );

	/* following were block data logic */
	rt.lgStarkON = true;
	/*rt.nLineContShield = LINE_CONT_SHIELD_PESC;*/
	/* >>chng 04 jun 12, change from pesc to federman form of shielding function */
	rt.nLineContShield = LINE_CONT_SHIELD_FEDERMAN;

	trace.lgSecIon = false;
	trace.lgTrOvrd = true;
	trace.lgTraceInput = false;
	trace.lgOpacBug = false;
	trace.nTrConvg = 0;
	trace.lgTr8446 = false;
	trace.lgTrLevN = false;
	trace.lgTrGant = false;
	trace.lgOptcBug = false;
	trace.lgTrace3Bod = false;
	trace.lgOTSBug = false;
	trace.lgESOURCE = false;
	trace.lgTr_CO_Mole = false;
	trace.lgTr_H2_Mole = false;
	trace.lgHeatBug = false;
	trace.lgHeavyBug = false;
	trace.lgBug2nu = false;
	trace.lgDrBug = false;
	trace.lgPtrace = false;
	trace.lgDrv_cdLine = false;
	trace.lgDustBug = false;
	trace.lgComBug = false;
	trace.lgHeBug = false;
	trace.lgCarBug = false;
	trace.lgCalBug = false;
	trace.lgConBug = false;
	trace.lgNeBug = false;
	trace.lgFeBug = false;
	trace.lgHBug = false;
	trace.lgTrLine = false;
	trace.nznbug = 10000;
	trace.npsbug = 10000;
	trace.lgTrace = false;
	trace.lgPointBug = false;
	trace.lgNeonBug = false;
	trace.lgCoolTr = false;
	trace.lgTrDiff = false;
	trace.lgIsoTraceFull[ipH_LIKE] = false;
	trace.lgIsoTraceFull[ipHE_LIKE] = false;

	/* parameters dealing with printer plots */
	for( i=0; i < NDPLOT; i++ )
	{
		plotCom.lgPltTrace[i] = false;
	}

	/* this says what types of printer plots we will make */
	for( i=0; i < NDPLOT; i++ )
	{
		strcpy( plotCom.chPType[i], "NONE" );
	}
	plotCom.lgPlotON = false;

	DEBUG_EXIT( "Zerologic()" );
	return;
}
